/* KallistiOS 1.1.8

   nehe06.c
   (c)2001 Benoit Miller

   Parts (c)2000 Jeff Molofee
   Parts (c)2003 Wayne Moorefield
*/

#include <dcos.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <pcx/pcx.h>

/* Simple KGL example to demonstrate texturing. 

   Essentially the same thing as NeHe's lesson06 code.
   To learn more, go to http://nehe.gamedev.net/.
*/

GLfloat xrot;	/* X Rotation */
GLfloat yrot;	/* Y Rotation */
GLfloat zrot;	/* Z Rotation */

GLuint texture[1];	/* Storage For One Texture */

/* Load a texture using pcx_load_texture and glKosTex2D */
void loadtxr(const char *fn, GLuint *txr) {
	kos_img_t img;
	pvr_ptr_t txaddr;

	if (pcx_to_img(fn, &img) < 0) {
		printf("can't load %s\n", fn);
		return;
	}

	txaddr = pvr_mem_malloc(img.w * img.h * 2);
	pvr_txr_load_kimg(&img, txaddr, PVR_TXRLOAD_INVERT_Y);
	kos_img_free(&img, 0);

	glGenTextures(1, txr);
	glBindTexture(GL_TEXTURE_2D, *txr);
	glKosTex2D(GL_RGB565_TWID, img.w, img.h, txaddr);
}

void draw_gl(void) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	glTranslatef(0.0f,0.0f,-5.0f);

	glRotatef(xrot,1.0f,0.0f,0.0f);
	glRotatef(yrot,0.0f,1.0f,0.0f);
	glRotatef(zrot,0.0f,0.0f,1.0f);

	glBindTexture(GL_TEXTURE_2D, texture[0]);

	glBegin(GL_QUADS);
		/* Front Face */
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		/* Back Face */
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		/* Top Face */
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		/* Bottom Face */
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		/* Right face */
		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
		/* Left Face */
		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
		glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
		glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
	glEnd();

	xrot+=0.3f;
	yrot+=0.2f;
	zrot+=0.4f;
}

pvr_init_params_t params = {
        /* Enable opaque and translucent polygons with size 16 */
        { PVR_BINSIZE_16, PVR_BINSIZE_0, PVR_BINSIZE_16, PVR_BINSIZE_0, PVR_BINSIZE_0 },

        /* Vertex buffer size 512K */
        512*1024
};

int PollJoystick(JoystickState *jState)
{
	static int state = 0;
	static HANDLE	joystick;
	int retval = DCI_OK;

	switch(state) {
	case 0:
		retval = OpenJoystick(&joystick, "/dev/joysticka");
		if(retval != DCI_OK) {
			state = 0;
			break;
		}
/*
		retval  = SetJoystickProperty(joystick, JOY1_DEADZONE_X, 100);
		retval |= SetJoystickProperty(joystick, JOY1_DEADZONE_Y, 100);
		retval |= SetJoystickProperty(joystick, JOY2_DEADZONE_X, 100);
		retval |= SetJoystickProperty(joystick, JOY2_DEADZONE_Y, 100);
		retval |= SetJoystickProperty(joystick, JOY1_SATURATION_X, 100);
		retval |= SetJoystickProperty(joystick, JOY1_SATURATION_Y, 100);
		retval |= SetJoystickProperty(joystick, JOY2_SATURATION_X, 100);
		retval |= SetJoystickProperty(joystick, JOY2_SATURATION_Y, 100);
*/
		if(retval != DCI_OK) {
			CloseJoystick(joystick);
			state = 0;
			break;
		}

		state = 1;
	case 1:
		retval = GetJoystickState(joystick, jState);
		if(retval != DCI_OK) {
			CloseJoystick(joystick);
			state = 0;
			break;
		}

		state = 1;
		break;
	}

	return retval;
}

int main(int argc, char **argv) {
	JoystickState	joyState;

	/* Initialize KOS */
	pvr_init(&params);

	printf("nehe06 beginning\n");

	/* Get basic stuff initialized */
	glKosInit();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f,640.0f/480.0f,0.1f,100.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 0.5f);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	/* Set up the texture */
	loadtxr("/rd/NeHe.pcx", &texture[0]);

	while(1) {
		/* Check key status */
		if(PollJoystick(&joyState) == DCI_OK) {
			if(joyState.buttons & JOY_START) {
				break;
			}
		}

		/* Begin frame */
		glKosBeginFrame();

		/* Draw the GL "scene" */
		draw_gl();

		/* Finish the frame */
		glKosFinishFrame();
	}

	return 0;
}


