
#include <dev/video.h>
#include <dev/biosfont.h>
#include <stdlib.h>
#include <dcos.h>

void DrawJoystickState(int where, JoystickState *state);
void DrawMouseState(int where, MouseState *state);
void DrawKeyboardState(int where, KeyboardState *state);

int PollJoystick(JoystickState *jState);
int PollMouse(MouseState *mState);
int PollKeyboard(KeyboardState *kState);

#define TEXT_LINE_SIZE		(640*24)

#define PORTA_LINE_START	(20*640+20)
#define PORTA_TYPE		(PORTA_LINE_START + TEXT_LINE_SIZE)
#define PORTA_INFO_LINE1	(PORTA_TYPE + TEXT_LINE_SIZE)
#define PORTA_INFO_LINE2	(PORTA_INFO_LINE1 + TEXT_LINE_SIZE)
#define PORTA_LINE_END		(PORTA_INFO_LINE2)

#define PORTB_LINE_START	(PORTA_LINE_END + TEXT_LINE_SIZE)
#define PORTB_TYPE		(PORTB_LINE_START + TEXT_LINE_SIZE)
#define PORTB_INFO_LINE1	(PORTB_TYPE + TEXT_LINE_SIZE)
#define PORTB_INFO_LINE2	(PORTB_INFO_LINE1 + TEXT_LINE_SIZE)
#define PORTB_LINE_END		(PORTB_INFO_LINE2)

#define PORTC_LINE_START	(PORTB_LINE_END + TEXT_LINE_SIZE)
#define PORTC_TYPE		(PORTC_LINE_START + TEXT_LINE_SIZE)
#define PORTC_INFO_LINE1	(PORTC_TYPE + TEXT_LINE_SIZE)
#define PORTC_INFO_LINE2	(PORTC_INFO_LINE1 + TEXT_LINE_SIZE)
#define PORTC_LINE_END		(PORTC_INFO_LINE2)

#define PORTD_LINE_START	(PORTC_LINE_END + TEXT_LINE_SIZE)
#define PORTD_TYPE		(PORTD_LINE_START + TEXT_LINE_SIZE)
#define PORTD_INFO_LINE1	(PORTD_TYPE + TEXT_LINE_SIZE)
#define PORTD_INFO_LINE2	(PORTD_INFO_LINE1 + TEXT_LINE_SIZE)
#define PORTD_LINE_END		(PORTD_INFO_LINE2)

/*
 * Enables Deadzone for joysticks
 * Set 1 to enable
 */
#define	ENABLE_DEADZONE	0
#define DEADZONE	100

static void DrawString(uint32 where, char *text)
{
	bfont_draw_str(VRAMs + where, 640, 1, text);
}

static void myCallback(void *param)
{
	HANDLE	handle = (HANDLE)param;

	/*
	 * Signal the semaphore
	 */
	SemaphoreSignal(handle);

	/*
	 * Lets make this periodic
	 */
	CreateRealTimer(myCallback, handle, 16);
}

/*
 * Your program's main entry point
 */
int main(int argc, char **argv)
{
	int o;
	HANDLE	mySema;

	JoystickState	joyState;
	MouseState	mouseState;
	KeyboardState	keyboardState;

	o = 20*640 + 20;

	/* Test with ISO8859-1 encoding */
	bfont_set_encoding(BFONT_CODE_ISO8859_1);

	/*
	 * Create the semaphore
	 */
	mySema = CreateSemaphore(0, 1, "mySema");

	/*
	 * Create a timeout for 1/60 of a second 16.6 ms
	 */
	CreateRealTimer(myCallback, mySema, 16);

	while(1) {
		SemaphoreWait(mySema, 0);

		DrawString(PORTA_LINE_START, "PORTA");
		if(PollJoystick(&joyState) == DCI_OK) {
			DrawString(PORTA_TYPE, "JOYSTICK CONNECTED");
			DrawJoystickState(o, &joyState);
		} else {
			DrawString(PORTA_TYPE, "JOYSTICK NOT FOUND");
		}
		
		DrawString(PORTB_LINE_START, "PORTB");

		DrawString(PORTC_LINE_START, "PORTC");
		if(PollKeyboard(&keyboardState) == DCI_OK) {
			DrawString(PORTC_TYPE, "KEYBOARD CONNECTED");
			DrawKeyboardState(o + TEXT_LINE_SIZE*6, &keyboardState);
		} else {
			DrawString(PORTD_TYPE, "KEYBOARD NOT FOUND");
		}

		DrawString(PORTD_LINE_START, "PORTD");
		if(PollMouse(&mouseState) == DCI_OK) {
			DrawString(PORTD_TYPE, "MOUSE CONNECTED");
			DrawMouseState(o + TEXT_LINE_SIZE*3, &mouseState);
		} else {
			DrawString(PORTD_TYPE, "MOUSE NOT FOUND");
		}
	}

	DeleteSemaphore(mySema);

	return 0;
}

int PollJoystick(JoystickState *jState)
{
	static int state = 0;
	static HANDLE	joystick;
	int retval = DCI_OK;

	switch(state) {
	case 0:
		retval = OpenJoystick(&joystick, "/dev/joysticka");
		if(retval != DCI_OK) {
			state = 0;
			break;
		}

#if ENABLE_DEADZONE
		retval  = SetJoystickProperty(joystick, JOY1_DEADZONE_X, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY1_DEADZONE_Y, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY2_DEADZONE_X, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY2_DEADZONE_Y, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY1_SATURATION_X, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY1_SATURATION_Y, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY2_SATURATION_X, DEADZONE);
		retval |= SetJoystickProperty(joystick, JOY2_SATURATION_Y, DEADZONE);
#endif /* ENABLE_DEADZONE */
		if(retval != DCI_OK) {
			CloseJoystick(joystick);
			state = 0;
			break;
		}

		state = 1;
	case 1:
		retval = GetJoystickState(joystick, jState);
		if(retval != DCI_OK) {
			CloseJoystick(joystick);
			state = 0;
			break;
		}

		state = 1;
		break;
	}

	return retval;
}

int PollMouse(MouseState *mState)
{
	static int state = 0;
	static HANDLE	mouse;
	int retval = DCI_OK;

	switch(state) {
	case 0:
		retval = OpenMouse(&mouse, "/dev/moused");
		if(retval != DCI_OK) {
			state = 0;
			break;
		}

		state = 1;
	case 1:
		retval = GetMouseState(mouse, mState);
		if(retval != DCI_OK) {
			CloseMouse(mouse);
			state = 0;
			break;
		}

		state = 1;
		break;
	}

	return retval;
}

int PollKeyboard(KeyboardState *kState)
{
	static int state = 0;
	static HANDLE	keyboard;
	int retval = DCI_OK;

	switch(state) {
	case 0:
		retval = OpenKeyboard(&keyboard, "/dev/keyboardc");
		if(retval != DCI_OK) {
			state = 0;
			break;
		}

		state = 1;
	case 1:
		retval = GetKeyboardState(keyboard, kState);
		if(retval != DCI_OK) {
			CloseKeyboard(keyboard);
			state = 0;
			break;
		}

		state = 1;
		break;
	}

	return retval;
}

void DrawJoystickState(int where, JoystickState *state)
{
	static char buffer1[128];
	static char buffer2[128];

	sprintf(buffer1, "Joy1X: %04d Joy1Y: %04d Joy2X: %04d Joy2Y: %04d",
		state->joy1x, state->joy1y, state->joy2x, state->joy2y);
	sprintf(buffer2, "Buttons: %04x LT: %03d RT: %03d",
		state->buttons, state->leftTrigger, state->rightTrigger);

	DrawString(PORTA_INFO_LINE1, buffer1);
	DrawString(PORTA_INFO_LINE2, buffer2);
}

void DrawMouseState(int where, MouseState *state)
{
	static char buffer1[128];
	static char buffer2[128];

	sprintf(buffer1, "X: %04d Y: %04d Z: %04d", state->dx, state->dy, state->dz);
	sprintf(buffer2, "Buttons: %08x", state->buttons);

	DrawString(PORTD_INFO_LINE1, buffer1);
	DrawString(PORTD_INFO_LINE2, buffer2);
}

void DrawKeyboardState(int where, KeyboardState *state)
{
	static char buffer1[128];

	for(int i=0; i<256; i++) {
		if(state->matrix[i]) {
			sprintf(buffer1, "Last Key Hit: %c", Keymap[0][i]);
			DrawString(PORTC_INFO_LINE1, buffer1);
		}
	}
}

