/*
 * Basic PCM Wave Playing
 */

#include <dcos.h>

#include <dev/video.h>
#include <dev/biosfont.h>
#include <sys/fs.h>
#include <swlog.h>
#include <input.h>

#include <stdlib.h>

#define BUFFERSIZE	0x15888 /*AC48 */

int PollJoystick(JoystickState *jState)
{
	static int state = 0;
	static HANDLE	joystick;
	int retval = DCI_OK;

	switch(state) {
	case 0:
		retval = OpenJoystick(&joystick, "/dev/joysticka");
		if(retval != DCI_OK) {
			state = 0;
			break;
		}

		state = 1;
	case 1:
		retval = GetJoystickState(joystick, jState);
		if(retval != DCI_OK) {
			CloseJoystick(joystick);
			state = 0;
			break;
		}

		state = 1;
		break;
	}

	return retval;
}

static void myCallback(void *param)
{
	HANDLE	handle = (HANDLE)param;

	/*
	 * Lets make this periodic
	 */
	timeout(myCallback, handle, 50);

	/*
	 * Signal the semaphore
	 */
	SemaphoreSignal(handle);
}

static void MusicThread(void *param)
{
	HANDLE	wavFile;
	HANDLE	device;
	HANDLE	buffer;
	uint8	*address;

	uint32	bytesRead;

	struct WAVEFORMATEX	*pwfx;
	struct CHUNKINFO	parent;
	struct CHUNKINFO	child;

	uint32	offset = 0;
	uint32	lastPos = 0;
	uint32	startOffset;
	uint32  midPoint;

	HANDLE	sema;


	if(WaveOpenFile("/rd/test44k16bm.wav", &wavFile, &pwfx, &parent)) {
		swlog("test.wav not found", (uint32)wavFile, 0, 0, 0);
		return;
	}

	if(CreateSoundBuffer(&buffer, BUFFERSIZE, pwfx)) {
		swlog("CreateSoundBuffer failed", (uint32)buffer, 0, 0, 0);
		return;
	}

	if(LockSoundBuffer(buffer, (void**)&address)) {
		swlog("LockSoundBuffer failed", 0, 0, 0, 0);
		return;
	}

	midPoint = BUFFERSIZE/2;
	WaveStartDataRead(&wavFile, &child, &parent);
	WaveFileRead(&wavFile, BUFFERSIZE, address, &child, &bytesRead);

	UnlockSoundBuffer(buffer);

	if(PlaySoundBuffer(buffer)) {
		swlog("PlaySoundBuffer failed", 0, 0, 0, 0);
		return;
	}

	/*
	 * Create semaphore for filling in buffer
	 */
	sema = CreateSemaphore(0, 0, "NOTIFY");

	/*
	 * Create timeout when we need to fill buffer
	 * .5 seconds, half of the buffer
	 */
	timeout(myCallback, sema, 50);

	/*
	 * Signal we are init'ed
	 */
	SemaphoreSignal((HANDLE)param);

	while(1) {
		/*
		 * Wait until it's time to update buffer
		 */
		SemaphoreWait(sema, 0);

		/*
		 * Get position
		 */
		GetSoundBufferPosition(buffer, &offset);

#if 0
		swlog("Position", offset, lastPos, midPoint, 0);
#endif
		/*
		 * Fill in the appropriate part
		 */
		if((offset >= midPoint && lastPos < midPoint) || (offset < lastPos)) {
#if 0
			if(offset >= midPoint) {
				startOffset = 0;
			} else {
				startOffset = midPoint;
			}
#else
			startOffset = (offset >= midPoint) ? 0 : (midPoint);
#endif

#if 0
			swlog("WaveFileRead", lastPos, offset, startOffset, midPoint);
#endif
                        WaveFileRead(&wavFile, midPoint, address+startOffset, &child, &bytesRead);
			if(bytesRead < midPoint) {
				/* EOF */
#if 0
				swlog("EOF", bytesRead, midPoint, lastPos, offset);
#endif
				WaveStartDataRead(&wavFile, &child, &parent);
				WaveFileRead(&wavFile, midPoint-bytesRead, address+startOffset+bytesRead, &child, &bytesRead);
			}
		}

		lastPos = offset;
	}

	return;
}

int main(int argc, char **argv)
{
	int o;
	char strbuf[128];

	HANDLE	sema;
	HANDLE	thread;

	JoystickState	joyState;

	o = 20*640 + 20;

	/* Test with ISO8859-1 encoding */
	bfont_set_encoding(BFONT_CODE_ISO8859_1);
#if 1
	/*
	 * Create semaphore
	 * Create music thread
	 * Wait until music thread inits
	 * Delete semaphore
	 */
	sema = CreateSemaphore(0, 1, "SoundStart");
	thread = CreateThread("Music", 8192, MusicThread, sema);
	SemaphoreWait(sema, 0);
	DeleteSemaphore(sema);
#endif
	while(1) {
		/*
		 * Game Engine
		 */
		Sleep(100);
	}
}
